@@ -15,6 +15,7 @@ from .logicerror import LogicError
 SERVER_VERSION = '\n5.1.23-feihuroger\x00'
 
 TYPENO = {
+    'decimal':  0x00,  # FIELD_TYPE_DECIMAL
     'int':      0x03,  # FIELD_TYPE_LONG
     'long':     0x03,  # FIELD_TYPE_LONG
     'float':    0x05,  # FIELD_TYPE_DOUBLE
@@ -38,7 +39,7 @@ class RPCHandler(object):
         self.aclmap = server.aclmap
 
         self.buf = ""
-        self.packetheader = False
+        self.packetfull = False
         self.authed = False
         self.sid = -1
         self.datalist = []
@@ -57,23 +58,26 @@ class RPCHandler(object):
     def work(self):
         """
             处理socket的所有请求，一直执行,直到断掉连接
-            deal with the request of the socket, run forever until socket be breaked
+            deal with all requests of one socket, run forever until the socket be breaked
         """
         self._structpacket(self._handshake())
         self._sendall(''.join(self.datalist))
-        self.log.info('%-8s: %s ' % ('conBEGIN',  self.client_address))
+        # client socket is connecting
+        #self.log.info('%-8s: %s ' % ('conBEGIN',  self.client_address))
         self.state['tcC'] += 1
         connclose = 0
         try:
             while self.socket is not None:
                 data = self.socket.recv(16777216)
                 if not len(data):
-                    self.log.info('%-8s: %s ' % ('conERR',  self.client_address))
+                    self.log.warning('%-8s: %s ' % ('conERR',  self.client_address))
                     break
                 self._data_received(data)
         except socket.error, e:
             if e[0] == errno.EBADF:
-                self.log.info('%-8s: %s ' % ('conCLOS1',  self.client_address))
+                # 客户端socket 关闭
+                # client socket closed
+                # self.log.info('%-8s: %s ' % ('conCLOS1',  self.client_address))
                 self.state['ncC'] += 1
                 connclose = 1
         except ValueError:
@@ -87,11 +91,23 @@ class RPCHandler(object):
             self.__dict__.pop('socket', None)
             if connclose == 0:
                 # 链接非正常中断
-                self.log.info('%-8s: %s ' % ('conCLOS2',  self.client_address))
+                # the socket connection unusually closed
+                self.log.warning('%-8s: %s ' % ('conCLOS2',  self.client_address))
                 self.state['ecC'] += 1
 
     def _sendall(self, data):
-        self.socket.send(data)
+        # 分包发送(不知什么愿意,有时在某些操作系统环境下,如果data超长,某些平台客户端会接收不全数据,卡在那里)
+        # splite long data to some buffs, if data to long, some OS client can't receive all, will be hang up.
+        ## self.socket.send(data)
+        buffsize = 10240
+        datalen = len(data)
+        countbuff = int(datalen/buffsize)
+        modbuff = datalen%buffsize
+        if( countbuff > 0):
+            for x in xrange(0,countbuff):
+                self.socket.send(data[x*buffsize: (x+1)*buffsize])
+        if(modbuff > 0):
+            self.socket.send(data[countbuff*buffsize:])
         self.datalist = []
 
     def _structpacket(self, pkt):
@@ -216,18 +232,18 @@ class RPCHandler(object):
         self.buf += data
         if len(self.buf) >= 16777216:  # mysql 规范长度 16777216
             self.socket.close()
-        if not self.authed and len(self.buf) > 16777216:
+        if not self.authed and len(self.buf) >= 16777216:
             self.socket.close()
-        if not self.packetheader:
+        if not self.packetfull:
             if len(self.buf) >= 4:
                 len1, len2, self.sid = struct.unpack("<HBB", self.buf[:4])
                 length = (len2 << 16) + len1
-                self.packetheader = True
             else:
                 return
 
         body = self.buf[4:]
         if len(body) >= length:
+            self.packetfull = True
             cmdarg = body[:length]
             self.cmdarg = cmdarg
             self.buf = body[length:]
@@ -237,7 +253,7 @@ class RPCHandler(object):
                     self._query(cmdarg)
                 else:
                     self.auth(cmdarg)
-                self.packetheader = False
+                self.packetfull = False
                 self._sendall(''.join(self.datalist))
             except:
                 self._struct_error(9999, "HY000", "Internal server error. Close connection.")
@@ -300,7 +316,7 @@ class RPCHandler(object):
                     self.authed = True
                     self.username = username
                     (_, self.gdict) = self.aclmap[username]
-                    self.log.info('%-8s: %s@%s ' % ('authOK', username, self.client_address))
+                    #self.log.info('%-8s: %s@%s ' % ('authOK', username, self.client_address))
                 else:
                     self._structpacket('\xfe')
                     self.sid += 1
@@ -362,6 +378,8 @@ class RPCHandler(object):
                     collist.append((colname, 'datetime'))
                 elif celltype.__name__ == "bytearray":
                     collist.append((colname, 'buffer'))
+                elif cell.__class__.__name__ == 'Decimal':
+                    collist.append((colname, 'decimal'))
                 else:
                     raise ValueError("cell type can not turn into mysql: %s" % repr(cell))
         else:
@@ -392,7 +410,7 @@ class RPCHandler(object):
 
             query = query.lower()
             if query == 'call':  # 存储过程调用
-                if param != 'pymysqlrpcinfo':
+                if param != 'pymysqlrpcinfo' and param != 'turnonlog' and param != 'turnofflog':
                     try:
                         self.state['tqC'] += 1
                         self.lastqueryBEGtime = time.time()
@@ -409,7 +427,8 @@ class RPCHandler(object):
 
                         self.lastqueryENDtime = time.time()
                         request_time = 1000.0 * (self.lastqueryENDtime - self.lastqueryBEGtime)
-                        self.log.info("%-8s: %s@%s:%.2fms:%s" % ('callOK1', self.username, self.client_address, request_time, param))
+                        if self.server.frameworklog:
+                            self.log.info("%-8s: %s@%s:%10.03f s:%8.03fms:%s" % ('callOK1', self.username, self.client_address, self.lastqueryBEGtime, request_time, param))
                         if not retvar:
                             self._struct_ok(1, 0, 0, 0, "")
                         else:
@@ -419,20 +438,31 @@ class RPCHandler(object):
                         # 逻辑错误，也是我们主程序用在正确执行了过程，只是返回了错误结果中，所以也要记录 info
                         self.lastqueryENDtime = time.time()
                         request_time = 1000.0 * (self.lastqueryENDtime - self.lastqueryBEGtime)
-                        self.log.info("%-8s: %s@%s:%.2fms:%s" % ('callOK2', self.username, self.client_address, request_time, param))
+                        if self.server.frameworklog:
+                            self.log.info("%-8s: %s@%s:%10.03f s:%8.03fms:%s" % ('callOK2', self.username, self.client_address, self.lastqueryBEGtime, request_time, param))
                         self._struct_error(ex.errno, "HY100", ex.errmsg)
                     except Exception, ex:
                         self.lastqueryENDtime = time.time()
                         self._struct_error(500, "HY102", "func call 2: --"+str(ex)+"--:" + param[:100])
-                        self.log.error('%-8s: %s@%s:%s' % ('callBAD2', self.username, self.client_address, param))
+                        self.log.error('%-8s: %s@%s:%10.03f s:%s' % ('callBAD2', self.username, self.client_address, self.lastqueryBEGtime, param))
                         self.state['eqC'] += 1
                     finally:
                         pass
                     return
 
-                elif self.username == 'root':  # only root can "call pymysqlrpcinfo;"  to get server infomation
+                elif self.username == 'root':
+                    # only root can "call pymysqlrpcinfo;"  to get server infomation
+                    # logon: turn on log of info
+                    # logoff: turn off log fo info
                     # get pymysqlrpc server running status
-                    collist, dataset = self._toresultset(self.server.serverinfo())
+                    if param == 'pymysqlrpcinfo':
+                        collist, dataset = self._toresultset(self.server.serverinfo())
+                    else:
+                        if param == 'turnonlog':
+                            self.server.turnonlog()
+                        elif param == 'turnofflog':
+                            self.server.turnofflog()
+                        collist, dataset =self._toresultset((('frameworklog',),[(str(self.server.frameworklog),)]))
                     self._struct_resultset(collist, dataset)
                     return
                 else:
@@ -450,7 +480,7 @@ class RPCHandler(object):
 
         elif cmd == '\x01':  # mysql cmd quit;
             self.socket.close()
-            self.log.info('%-8s: %s@%s ' % ('authCLOS', self.username, self.client_address))
+            #self.log.info('%-8s: %s@%s ' % ('authCLOS', self.username, self.client_address))
             return
         elif cmd == '\x02':  # use somedb;
             self._struct_simpleok()
